#include <stdio.h>
#include <stdlib.h>

#include "../headers/bmp_reader.h"

enum read_status from_bmp(FILE *file, struct image *img) {
    struct bmp_header *header = calloc(1, sizeof(struct bmp_header));
    if (!header) {
        return MEMORY_ALLOCATION_ERROR;
    }

    size_t data_readed = fread(header, sizeof(struct bmp_header), 1, file);
    if (data_readed != 1) {
        free(header);
        return READ_INVALID_HEADER;
    }

    if (header->bfType != 0x4D42) {
        free(header);
        return READ_INVALID_SIGNATURE;
    }

    if (header->biBitCount != 24) {
        free(header);
        return READ_BIT_COUNT_ERROR;
    }

    fseek(file, header->bOffBits, SEEK_SET);

    int row_size = ((header->biBitCount * header->biWidth + 31) / 32) * 4;
    img->height = header->biHeight;
    img->width = header->biWidth;
    img->data = calloc(img->width * img->height, sizeof(struct pixel));
    if (!img->data) {
        free(header);
        return MEMORY_ALLOCATION_ERROR;
    }

    for (size_t i = 0; i < header->biHeight; i++) {
        size_t data = fread(&img->data[i * img->width], sizeof(struct pixel), img->width, file);
        if (data != img->width) {
            free(header);
            free(img->data);
            return READ_INVALID_BITS;
        }

        if (row_size > img->width * sizeof(struct pixel)) { //пропуск паддинга
            fseek(file, row_size - img->width * sizeof(struct pixel), SEEK_CUR);
        }
    }

    free(header);
    return READ_OK;
}
