#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "headers/bmp_reader.h"
#include "headers/bmp_writer.h"
#include "headers/errors.h"
#include "headers/transformation.h"
#include "headers/util.h"

int main(int argc, char **argv){
    if (argc != 4){
        fprintf(stderr, "Неправильное использование!\nПример правильного использования:\n%s oleg.bmp oleg2.bmp none\n", argv[0]);
        exit(1);
    }

    //чтение входного изображения
    FILE *file;
    enum open_status open_read_status = open_read(argv[1], &file);
    check_open_errors(open_read_status);

    struct image *durov = malloc(sizeof(struct image));
    enum read_status read_status = from_bmp(file, durov);
    check_read_errors(read_status);
    
    enum close_status close_read_status = close(&file);
    check_close_errors(close_read_status);

    //обработка изображения
    struct image *res_image = malloc(sizeof(struct image));

    if (!strcmp(argv[3], "none")){
        *res_image = do_nothing(*durov);
    }
    else if (!strcmp(argv[3], "cw90")){ // по часовой на 90
        *res_image = rotate_90_right(*durov);
    }
    else if (!strcmp(argv[3], "ccw90")){ // против часовой на 90
        *res_image = rotate_90_left(*durov);
    }
    else if (!strcmp(argv[3], "fliph")){ // отразить по горизонтали
        *res_image = swap_horizontal(*durov);
    }
    else if (!strcmp(argv[3], "flipv")){ // отразить по вертикали
        *res_image = swap_vertical(*durov);
    }
    else{
        fprintf(stderr, "ERROR: Неизвестная команда трансформации\n");
        exit(22);
    };
    
    //запись в выходное изображение
    FILE *res_file;
    enum open_status open_write_status = open_write(argv[2], &res_file);
    check_open_errors(open_write_status);

    enum write_status write_status = to_bmp(res_file, res_image);
    check_write_errors(write_status);

    enum close_status close_write_status = close(&res_file);
    check_close_errors(close_write_status);

    free(durov->data);
    free(durov);
    free(res_image->data);
    free(res_image);

    exit(0);
}
